#!/bin/bash

LDIF=$(mktemp)
PWDSCRIPT=$(mktemp)
DRYRUN=0
ENCODE=0

function usage()
{
   echo "usage: $(basename $0) -i inputfile.txt inputfile.txt -b /var/samba/dbfile.ldb"
   echo "     options : -d dry run don't do anything and don't cleanup temporary files"
   echo "     options : -e enable base64 encoding of the objectSID"
   return 0
}

while getopts :i:b:hde FLAG; do
  case $FLAG in
    i)  #set option "i"
      INPUT=$OPTARG
      ;;
    b)  #set option "b"
      DBFILE=$OPTARG
      ;;
	e) ENCODE=1 ;;
	d) DRYRUN=1 ;;
    h)  #show help
      usage
      exit 1
      ;;
    \?) #unrecognized option - show help
      echo -e \\n"Option -${BOLD}$OPTARG${NORM} not allowed."
      usage
      exit 2
      ;;
  esac
done

if [[ -z ${INPUT} ]]
then
	usage
	echo "Input files are mandatory"
	exit 1
fi

if [[ -z ${DBFILE} ]]
then
	usage
	echo "Database file is mandatory"
	exit 1
fi

for file in ${INPUT}
do
	echo -n "Working on ${file} "

	echo "Working on ${file} "

	while read -r line
	do
		data=(${line//;/ })
		machine=0
		[[ "${data[0]: -1}" == "$" ]] && machine=1

		if [[ ${machine} -eq 1 ]]
		then
			if [[ ${DRYRUN} -eq 0 ]]
			then
				pdbedit -a -m -u ${data[0]}
			else
				echo "pdbedit -a -m -u ${data[0]}"
			fi
		fi

		res=0
		if [[ ${DRYRUN} -eq 1 ]]
		then
			dn=$(samba-tool user getpassword ${data[0]} --attributes dn | head -1)
			if [[ $? -ne 0 ]] 
			then
				if [[ ${machine} -eq 1 ]]
				then
					dn="dn: ${data[0]} non existing machine account"
					res=0
				else
					res=1
				fi
			else
				res=0
			fi
		else
			dn=$(samba-tool user getpassword ${data[0]} --attributes dn | head -1)
			res=${?}
		fi

		if [[ ${res} -eq 0 ]]
		then
			pwd=${data[1]}
			if [[ ${ENCODE} -eq 1 ]]
			then
				sid=$(echo ${data[2]} | base64)
			else
				sid=${data[2]}
			fi

			if [[ -n ${dn} ]] && [[ -n ${sid} ]]
			then
				echo ${dn}
				echo -e "${dn}\nchangetype: modify\nreplace: objectSid\nobjectSid: ${sid}\n" >> ${LDIF}
				echo "Working on ${data[0]} user."
				echo "pdbedit -d 0 --set-nt-hash=${pwd} ${data[0]}" >> ${PWDSCRIPT}
			fi
		else
			echo "No dn found for user ${data[0]}"
		fi
	done < "${file}"
	echo
done

if [[ ${DRYRUN} -eq 0 ]]
then
	ldbmodify -v -H "${DBFILE}" ${LDIF}
	source ${PWDSCRIPT}

	rm -rf ${LDIF}
	rm -rf ${PWDSCRIPT}

else
	echo "/usr/bin/ldbmodify -v -H "${DBFILE}" ${LDIF}"
	cat ${PWDSCRIPT}
	echo "ldif file is : ${LDIF}"
	echo "password script is ${PWDSCRIPT}"
fi


